#!/usr/bin/php
<?php
/*
  check_ftp_speed.php
  Icinga / Nagios Plugin to test FTP speed by writing and reading a file and measuring the time taken
  Performance Data is supplied for Read and Write Speed in MBytes/s
  Warning and Critical limits can be set for Write Speed
  (C) GEOTEK Datentechnik Berlin, Martin Kasztantowicz
  http://geotek.de/ - info@geotek.de

  V.1.0
*/ 


$msg = '';
$err = '';
$return_code = 3;
$return_status = array(0 => 'OK', 1 => 'WARNING', 2 => 'CRITICAL', 3 => 'UNKNOWN');
// We must use double quotes to make Newline working on the commandline!
$usage = "Usage: checkftp_speed.php -H <host> -u <username> -p <password> -f <test file path> ".
  "-s <test file size (kBytes)> -w <warning (Write MBytes/s)> -c <critical (Write MBytes/s)>\n";
// $argstr = var_export($argv, true);

// Parse ommand line
$options = getopt('H:u:p:f:s:w:c:f:');

$host = (isset($options['H'])) ? $options['H'] : '';
if ($host == '') {
  // We must use double quotes to make Newline working on the commandline!
  $err .= "-H Host parameter missing\n";
}

$username = (isset($options['u'])) ? $options['u'] : '';
if ($username == '') {
  $err .= "-u Username parameter missing\n";
}

$password = (isset($options['p'])) ? $options['p'] : '';
if ($password == '') {
  $err .= "-p Password parameter missing\n";
}

$file_path = (isset($options['f'])) ? $options['f'] : '';
if ($file_path == '') {
  $err .= "-f test file path missing\n";
}

$file_size = (isset($options['s'])) ? $options['s'] : '5000';

$warning = (isset($options['w'])) ? $options['w'] : '';
if ($warning == '') {
  $err .= "-w Warning speed (MBytes/s) missing\n";
}

$critical= (isset($options['c'])) ? $options['c'] : '';
if ($critical == '') {
  $err .= "-c Critical speed (MBytes/s) missing\n";
}


if ($err != '') abort($err.$usage);

$data = '';
// Create variable with requested file size
$data = str_pad($data, $file_size * 1024, 'x');
$conn = @ftp_connect($host);
if (!$conn) abort('Connection to \''.$host.'\' failed');

if (!@ftp_login($conn, $username, $password)) abort('Login with username \''.$username.'\' and password \''.$password.'\' failed');

// Important when accessing FTP Servers via NAT or Firewall
ftp_pasv($conn, true);

// Read test file that had been written in previous cycle
// This avoids misleading read speed result from FTP server chaching or buffering
$time_start = microtime_float();
$filename_read = 'ftp://'.$username.':'.$password.'@'.$host.$file_path;

$handle = @fopen($filename_read, 'r');
if ($handle != false) {
  // Test file is found
  // avoid error in first check when there is no testfile present yet
  $contents = '';
  while (!feof($handle)) {
    $contents .= fread($handle, 8192);
  }
  $duration_read = microtime_float() - $time_start;
  $speed_read = $file_size / $duration_read / 1024;
  if (!@ftp_delete($conn, $file_path)) abort('Unable to delete test file \''.$file_path.'\'');
}
else {
  // No data yet (1st check)
  $speed_read = 0;
}

// Write test file
$time_start = microtime_float();
if (!ftp_put($conn, $file_path, 'data://text/plain;base64,' . base64_encode($data), FTP_BINARY)) {
  abort('Writing test file \''.$file_path.'\' failed');
}
$duration_write = microtime_float() - $time_start;
$speed_write = $file_size / $duration_write / 1024;

// Leave test file on server to be read in the next check cycle

ftp_close($conn);

if ($speed_write < $critical) $return_code = 2;
else if ($speed_write < $warning) $return_code = 1;
else $return_code = 0;

// echo 'Speed OK - '.round($speed_read, 2).' MByte/s Read, '.round($speed_write, 2).' MByte/s Write';
echo 'Speed '.$return_status[$return_code].' - '.round($speed_read, 2).'/'.round($speed_write, 2).' MByte/s Read/Write of '.$file_size.' kBytes'.
'|Read='.round($speed_read, 3).'MByte/s Write='.round($speed_write, 3).'MByte/s'.
';'.$warning.';'.$critical;

exit($return_code);


function microtime_float()
{
    list($usec, $sec) = explode(" ", microtime());
    return ((float)$usec + (float)$sec);
}


function abort($msg) {
  global $return_code, $return_status;
    echo $return_status[$return_code].' - '.$msg;
    exit($return_code);
}

?>
